#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

#define PIN 8 // Digital I/O Pin 8 has the LED strip connected via a 510R resistor 

// LED test and experiment sequence for Neopixel style WS2811/2812 Strips
// Example code from adafruit
// modified by A Horne 4-4-2016
// For the 3D Printed Project Snowflake
// www.richrap.blogspot.com
// @RichRap3D


// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(35, PIN, NEO_RGB + NEO_KHZ800);

// Project Snowflake sculpture has 30 RGB pixels - but we have a few spare to go above and hang below the sculpture
// Actual pixels for the sculpture are 2-31 (30) and a total of 35
//

// IMPORTANT: To reduce NeoPixel burnout risk, add 1000 uF capacitor across
// pixel power leads, add 300 - 500 Ohm resistor on first pixel's data input
// and minimize distance between Arduino and first pixel.  Avoid connecting
// on a live circuit...if you must, connect GND first.

void setup() {
  // This is for Trinket 5V 16MHz, you can remove these three lines if you are not using a Trinket

  // End of trinket special code


  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
}

void loop() {
  // Some example procedures showing how to display to the pixels:
  colorWipe(strip.Color(255, 255, 255), 0); // White
  colorWipe(strip.Color(0, 0, 0), 100); // White
  colorWipe(strip.Color(255, 255, 255), 40); // White
  colorWipe(strip.Color(127, 127, 125), 35); // White
  colorWipe(strip.Color(64, 64, 64), 30); // White
  colorWipe(strip.Color(32, 32, 32), 25); // White
  colorWipe(strip.Color(16, 16, 16), 20); // White
  colorWipe(strip.Color(12, 12, 12), 15); // White
  colorWipe(strip.Color(6, 6, 6), 10); // White
  colorWipe(strip.Color(0, 0, 0), 5); // White

  theaterChaseRainbow(5);
    
  colorWipe(strip.Color(255, 0, 0), 50); // Red
  colorWipe(strip.Color(0, 255, 0), 50); // Green
  colorWipe(strip.Color(0, 0, 255), 50); // Blue
//colorWipe(strip.Color(0, 0, 0, 255), 50); // White RGBW

 theaterChaseRainbow(10);
 
  // Send a theater pixel chase in...
  theaterChase(strip.Color(255, 255, 255), 50); // White
  theaterChase(strip.Color(127, 127, 127), 50); // White
  theaterChase(strip.Color(16, 16, 16), 50); // White
  theaterChase(strip.Color(255, 0, 0), 100); // Red
  theaterChase(strip.Color(0, 255, 0), 150); // Green
  theaterChase(strip.Color(0, 0, 255), 200); // Blue
  theaterChase(strip.Color(127, 127, 127), 50); // White
  
 // TheaterChaseUpdate(strip.Color(0, 127, 0), 50); // Green

  theaterChaseRainbow(15);

  rainbow(20); //fast 20ms rainbow
  rainbow(100); //slow 100ms rainbow
  rainbowCycle(5);
  rainbowCycle(10);
  rainbowCycle(15);
  rainbowCycle(20);
  rainbowCycle(25);
  rainbowCycle(30);
  theaterChaseRainbow(10);
  theaterChaseRainbow(25);
  theaterChaseRainbow(50);
  theaterChaseRainbow(75);
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(wait);
  }
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}


//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) {
  for (int j=0; j<10; j++) {  //do 10 cycles of chasing
    for (int q=0; q < 3; q++) {
      for (uint16_t i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, c);    //turn every third pixel on
      }
      strip.show();

      delay(wait);

      for (uint16_t i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < 3; q++) {
      for (uint16_t i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every third pixel on
      }
      strip.show();

      delay(wait);

      for (uint16_t i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
